#!/usr/local/bin/php
<?php

$no_http_headers = true;

/* display No errors */
error_reporting(0);

if (isset($config)) {
        include_once(dirname(__FILE__) . "/../lib/snmp.php");
}

if (!isset($called_by_script_server)) {
        include_once(dirname(__FILE__) . "/../include/global.php");
        include_once(dirname(__FILE__) . "/../lib/snmp.php");

        array_shift($_SERVER["argv"]);
}

#
# global declarations
#
global $snmp_hostname;
global $snmp_community;
global $snmp_version;
global $snmp_username;
global $snmp_password;
global $snmp_port;
global $snmp_timeout;
global $oid;
global $command;
global $command_opt;
global $commandargs;
global $cmdtool;
global $cmdoutput;
global $cmdoutputcount;
global $output;
global $outputcount;
global $snmp_procname;
global $snmp_procsize;

$commandargs = explode(",", $_SERVER['argv'][0]);
array_shift($_SERVER['argv']);

#
# make sure enough parameters were provided
#
if (count($commandargs) < 7) {

	echo ("failed: insufficient parameters provided\n");

	displaySyntax();

	exit(1);
}

#
# 1st parameter is always $snmp_hostname
#
$snmp_hostname = $commandargs[0];

if ($snmp_hostname == "") {

	echo ("failed: invalid hostname\n");

	displaySyntax();

	exit(1);
}

#
# 2nd parameter is always $snmp_community; testing is done after we have the version
#
$snmp_community = $commandargs[1];

#
# 3rd parameter is always $snmp_version
#
$snmp_version = $commandargs[2];

if (($snmp_version != 1) and ($snmp_version != 2) and ($snmp_version != 3)) {

	echo ("failed: \"$snmp_version\" is not a valid SNMP version\n");

	displaySyntax();

	exit(1);
}

if (($snmp_version != 3) and ($snmp_community == "")) {

	echo ("failed: invalid SNMP community string\n");

	displaySyntax();

	exit(1);
}

#
# 4th parameter is always $snmp_username
#
$snmp_username = $commandargs[3];

#
# 5th parameter is always $snmp_password
#
$snmp_password = $commandargs[4];

#
# 6th parameter is always $snmp_port
#
$snmp_port = $commandargs[5];

if (is_numeric($snmp_port) == FALSE) {

	echo ("failed: \"$snmp_port\" is not a numeric port number \n");

	displaySyntax();

	exit(1);
}

#
# 7th parameter is always $snmp_timeout
#
$snmp_timeout = $commandargs[6];

if (is_numeric($snmp_timeout) == FALSE) {

	echo ("failed: \"$snmp_timeout\" is not a numeric timeout value \n");

	displaySyntax();

	exit(1);
}

#
# 8th and 9th parameters is always $command plus options
#
if (isset($_SERVER['argv'])) {
	$command = $_SERVER['argv'][0];
	array_shift($_SERVER['argv']);
	if (isset($_SERVER['argv'])) {
		$command_opt = $_SERVER['argv'];
	}
}

#
# Cacti's SNMP timeout uses milliseconds, while PHP uses the value
# from Net-SNMP's header files, which is typically microseconds.
# Normalize by multiplying the timeout parameter value by 1000.
#
$snmp_timeout = ($snmp_timeout * 1000);

#
# define a nested array of the OIDs and their labels
#
$oid = array (
	array ("hrSWRunName", ".1.3.6.1.2.1.25.4.2.1.2"),
	array ("hrSWRunPath", ".1.3.6.1.2.1.25.4.2.1.4"),
	array ("hrSWRunPerfCPU", ".1.3.6.1.2.1.25.5.1.1.1"),
	array ("hrSWRunPerfMem", ".1.3.6.1.2.1.25.5.1.1.2"),
	array ("hrProcessorFrwID", ".1.3.6.1.2.1.25.3.3.1.1")
);

#
# execute the command and process the results array
#
$cmdoutputcount = 0;

#
# run the program
#

while ($cmdoutputcount < count($oid)) {

	$cmdoutput[$cmdoutputcount] = cacti_snmp_walk($snmp_hostname,
		$snmp_community,
		$oid[$cmdoutputcount][1],
		$snmp_version,
		$snmp_username,
		$snmp_password,
		"",
		"",
		"",
		"",
		$snmp_port,
		$snmp_timeout);

	$cmdoutputcount++;
}

if (count($cmdoutput) < count($oid)) {

	echo ("failed: too few results returned from $cmdtool");

	exit(1);
}

#
#
# put wanted OIDs to array
#

$cmdoutputcount = 0;
$procnameOids = array();
while (isset($cmdoutput[0][$cmdoutputcount]['value'])) {
	$procname = $cmdoutput[0][$cmdoutputcount]['value'];
	$procpath = $cmdoutput[1][$cmdoutputcount]['value'];
	if (isset($cmdoutput[2][$cmdoutputcount]['value'])) {
		$proccpu = $cmdoutput[2][$cmdoutputcount]['value'];
	} else {
		$proccpu = 0;
	}
	if (isset($cmdoutput[3][$cmdoutputcount]['value'])) {
		$procsize = $cmdoutput[3][$cmdoutputcount]['value'];
	} else {
		$procsize = 0;
	}

	if (!isset($procnameOids[$procname]['proccount'])) {
		$procnameOids[$procname]['proccount'] = 0;
	}
	if (!isset($procnameOids[$procname]['proccpu'])) {
		$procnameOids[$procname]['proccpu'] = 0;
	}
	if (!isset($procnameOids[$procname]['procpath'])) {
		$procnameOids[$procname]['procpath'] = "";
	}
	if (!isset($procnameOids[$procname]['procsize'])) {
		$procnameOids[$procname]['procsize'] = 0;
	}

	# make sure the data only contains numbers and sum them
	if (substr($procsize,-2) == "kB") {
		$procsize = intval($procsize);
	} elseif (substr($procsize,-6) == "KBytes") {
		$procsize = intval($procsize);
	} else {
		$procsize = intval($procsize);
	}

	$procnameOids[$procname]['proccount'] ++;
	$procnameOids[$procname]['proccpu'] += $proccpu;
	$procnameOids[$procname]['procpath'] = $procpath;
	$procnameOids[$procname]['procsize'] += $procsize;

	$cmdoutputcount++;
}

#
# generate output
#

ksort($procnameOids);

$cpu_count = 1;
if (isset($cmdoutput[4])); {
	$cpu_count = count($cmdoutput[4]);
}

if ($command == "num_indexes") {
	$output = count($procnameOids);
	echo ($output);
	exit(0);
}

$cmdoutputcount = 0;
foreach (array_keys($procnameOids) as $key => $value) {
	$procname = $value;
	$procpath = $procnameOids[$procname]['procpath'];
	$proccpu = round($procnameOids[$procname]['proccpu'] / $cpu_count);
	$procsize = $procnameOids[$procname]['procsize'];
	$proccount = $procnameOids[$procname]['proccount'];

	if ($command == "index") {
		$output = $procname . "\n";
	} elseif ($command == "query") {
		if (isset($command_opt)) {
			if ($command_opt[0] == "index") {
				$output = $procname . "!" . $procname . "\n";
			} elseif ($command_opt[0] == "hrSWRunName") {
				$output = $procname . "!" . $procname . "\n";
			} elseif ($command_opt[0] == "hrSWRunPath") {
				$output = $procname . "!" . $procpath . "\n";
			} elseif ($command_opt[0] == "hrProcessorFrwID") {
				$output = $procname . "!" . $cpu_count . "\n";
			} elseif ($command_opt[0] == $procname) {
				$output = $procname . "!" . $procname . "\n";
				echo ($output);
				break;
			}
		}
	} elseif ($command == "get") {
		if ($command_opt[1] == $procname) {
			if ($command_opt[0] == "procname") {
				$output = $procname . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "procpath") {
				$output = $procpath . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "proccpu") {
				$output = $proccpu . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "procsize") {
				$output = $procsize . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "proccount") {
				$output = $proccount . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "procname") {
				$output = $procname . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "hrSWRunName") {
				$output = $procname . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "hrProcessorFrwID") {
				$output = $cpu_count . "\n";
				echo ($output);
				break;
			} elseif ($command_opt[0] == "hrSWRunPath") {
				$output = $procpath . "\n";
				echo ($output);
				break;
			}
		}
	}

	echo ($output);
}

exit(0);

function displaySyntax() {

	#
	# display the syntax
	#
	echo ("syntax: " . $_SERVER['argv'][0] . " <hostname>, [<snmp_community>], <snmp_version>,\n" .
		"   [<snmp3_username>], [<snmp3_password>], <snmp_port>, <snmp_timeout>, <snmp_procname>\n");
}

?>

